/* Fo
 * fo-length-cond.c: 'length-conditional' datatype
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-length-conditional.c,v 1.4 2003/05/16 10:43:45 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include "libfo/fo-utils.h"
#include "fo-datatype.h"
#include "fo-datatype-private.h"
#include "fo-enum.h"
#include "fo-length.h"
#include "fo-length-conditional.h"


enum {
  PROP_0,
  PROP_LENGTH,
  PROP_CONDITY
};

struct _FoLengthCond
{
  FoDatatype parent_instance;

  FoDatatype *length;
  FoDatatype *condity;
};

struct _FoLengthCondClass
{
  FoDatatypeClass parent_class;
  
};

static void fo_length_cond_class_init   (FoLengthCondClass *klass);
static void fo_length_cond_set_property (GObject       *object,
				    guint          prop_id,
				    const GValue  *value,
				    GParamSpec    *pspec);
static void fo_length_cond_get_property (GObject       *object,
				    guint          prop_id,
				    GValue        *value,
				    GParamSpec    *pspec);
static void fo_length_cond_finalize     (GObject       *object);

gchar*      fo_length_cond_sprintf (FoObject *object);
FoDatatype* fo_length_cond_copy (FoDatatype *datatype);

static gpointer parent_class;

/**
 * fo_length_cond_get_type:
 * @void: 
 * 
 * Register the #FoLengthCond object type.
 * 
 * Return value: GType value of the #FoLengthCond object type.
 **/
GType
fo_length_cond_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoLengthCondClass),
        NULL,           /* base_init */
        NULL,           /* base_finalize */
        (GClassInitFunc) fo_length_cond_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoLengthCond),
        0,              /* n_preallocs */
        NULL,		/* instance_init */
	NULL		/* value_table */
      };
      
      object_type = g_type_register_static (FO_TYPE_DATATYPE,
                                            "FoLengthCond",
                                            &object_info, 0);
    }
  
  return object_type;
}

/**
 * fo_length_cond_class_init:
 * @klass: FoLengthCondClass object to initialise
 * 
 * Implements GClassInitFunc for FoLengthCondClass
 **/
void
fo_length_cond_class_init (FoLengthCondClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  
  parent_class = g_type_class_peek_parent (klass);
  
  object_class->finalize = fo_length_cond_finalize;

  object_class->set_property = fo_length_cond_set_property;
  object_class->get_property = fo_length_cond_get_property;

  FO_OBJECT_CLASS (klass)->sprintf = fo_length_cond_sprintf;

  FO_DATATYPE_CLASS (klass)->copy = fo_length_cond_copy;

  g_object_class_install_property (object_class,
                                   PROP_LENGTH,
                                   g_param_spec_object ("length",
							_("Length"),
							_("Length_Cond length"),
							FO_TYPE_DATATYPE,
							G_PARAM_READWRITE |
							G_PARAM_CONSTRUCT_ONLY));

  g_object_class_install_property (object_class,
                                   PROP_CONDITY,
                                   g_param_spec_object ("condity",
							_("Condity"),
							_("Length condity"),
							FO_TYPE_DATATYPE,
							G_PARAM_READWRITE |
							G_PARAM_CONSTRUCT_ONLY));

}

static void
fo_length_cond_finalize (GObject *object)
{
  FoLengthCond *length_cond;

  length_cond = FO_LENGTH_COND (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


static void
fo_length_cond_set_property (GObject         *object,
                         guint            prop_id,
                         const GValue    *value,
                         GParamSpec      *pspec)
{
  FoDatatype *length_cond;

  length_cond = FO_DATATYPE (object);

  switch (prop_id)
    {
    case PROP_LENGTH:
      fo_length_cond_set_length (length_cond, g_value_get_object (value));
      break;
    case PROP_CONDITY:
      fo_length_cond_set_condity (length_cond, g_value_get_object (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
fo_length_cond_get_property (GObject         *object,
                         guint            prop_id,
                         GValue          *value,
                         GParamSpec      *pspec)
{
  FoDatatype *length_cond;

  length_cond = FO_DATATYPE (object);

  switch (prop_id)
    {
    case PROP_LENGTH:
      g_value_set_object (value, fo_length_cond_get_length (length_cond));
      break;
    case PROP_CONDITY:
      g_value_set_object (value, fo_length_cond_get_condity (length_cond));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/**
 * fo_length_cond_new:
 * 
 * Creates a new #FoLengthCond initialized to default value.
 * 
 * Return value: the new #FoLengthCond
 **/
FoDatatype *
fo_length_cond_new (void)
{
  FoDatatype *length_cond = NULL;

  length_cond = FO_DATATYPE (g_object_new (fo_length_cond_get_type (),
					   "length",
					   fo_length_get_length_zero (),
					   "condity",
					   fo_datatype_get_condity_discard (),
					   NULL));
  
  return length_cond;
}

/**
 * fo_length_cond_new_with_value:
 * @length:  Length component of the new #FoLengthCond
 * @condity: Conditionality component of the new #FoLengthCond
 * 
 * Creates a new #FoLengthCond set to @length and @condity
 * 
 * Return value: The new #FoLengthCond
 **/
FoDatatype*
fo_length_cond_new_with_value (FoDatatype *length,
			       FoDatatype *condity)
{
  FoDatatype *length_cond = fo_length_cond_new ();

  g_return_val_if_fail (FO_IS_LENGTH (length), NULL);
  g_return_val_if_fail (condity == NULL ||
			FO_IS_ENUM (condity), NULL);

  fo_length_cond_set_length (length_cond, length);

  if (condity)
    {
      fo_length_cond_set_condity (length_cond, condity);
    }
  else
    {
      fo_length_cond_set_condity (length_cond,
				  fo_datatype_get_condity_discard ());
    }
  return length_cond;
}

/**
 * fo_length_cond_set_length:
 * @datatype:    #FoLengthCond
 * @new_length:  New .length value
 * 
 * Sets the .length component of @datatype
 **/
void
fo_length_cond_set_length (FoDatatype *datatype,
			   FoDatatype *new_length)
{
  FoLengthCond *length_cond = (FoLengthCond *) datatype;

  g_return_if_fail (length_cond != NULL);
  g_return_if_fail (FO_IS_LENGTH_COND (length_cond));
  g_return_if_fail (FO_IS_DATATYPE (new_length));

  if (new_length)
    g_object_ref (G_OBJECT (new_length));
  if (length_cond->length)
    g_object_unref (G_OBJECT (length_cond->length));
  length_cond->length = new_length;
  /*g_object_notify(G_OBJECT(length_cond), "length");*/
}

/**
 * fo_length_cond_get_length:
 * @datatype: #FoLengthCond
 * 
 * Gets the .length component value of @datatype
 * 
 * Return value: The .length value of @datatype
 **/
FoDatatype*
fo_length_cond_get_length (FoDatatype *datatype)
{
  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_LENGTH_COND (datatype), 0);

  return FO_LENGTH_COND (datatype)->length;
}

/**
 * fo_length_cond_set_condity:
 * @datatype:     #FoLengthCond
 * @new_condity:  New .condity value
 * 
 * Sets the .condity component of @datatype
 **/
void
fo_length_cond_set_condity (FoDatatype *datatype,
			    FoDatatype *new_condity)
{
  FoLengthCond *length_cond = (FoLengthCond *) datatype;

  g_return_if_fail (length_cond != NULL);
  g_return_if_fail (FO_IS_LENGTH_COND (length_cond));
  g_return_if_fail (FO_IS_DATATYPE (new_condity));

  if (new_condity)
    g_object_ref (G_OBJECT (new_condity));
  if (length_cond->condity)
    g_object_unref (G_OBJECT (length_cond->condity));
  length_cond->condity = new_condity;
  /*g_object_notify(G_OBJECT(length_cond), "conditionality");*/
}

/**
 * fo_length_cond_get_condity:
 * @datatype: #FoLengthCond
 * 
 * Gets the .conditionality component value of @datatype
 * 
 * Return value: The .conditionality component value of @datatype
 **/
FoDatatype*
fo_length_cond_get_condity (FoDatatype *datatype)
{
  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_LENGTH_COND (datatype), 0);

  return FO_LENGTH_COND (datatype)->condity;
}

/**
 * fo_length_cond_sprintf:
 * @object: #FoLengthCond whose value is to be printed
 * 
 * Create and return a string representation of the value of @object,
 * which must be an #FoLengthCond.
 *
 * String must be freed by caller.
 * 
 * Return value: String representation of value of @object
 **/
gchar*
fo_length_cond_sprintf (FoObject *object)
{
  g_return_val_if_fail (object != NULL, NULL);
  g_return_val_if_fail (FO_IS_LENGTH_COND (object), NULL);

  return g_strdup_printf ("length: %s; condity: %s",
			  fo_object_sprintf (FO_LENGTH_COND (object)->length),
			  fo_object_sprintf (FO_LENGTH_COND (object)->condity));
}

/**
 * fo_length_cond_copy:
 * @datatype: Source #FoLengthCond
 * 
 * Creates a copy of @datatype
 * 
 * Return value: Copy of @datatype
 **/
FoDatatype*
fo_length_cond_copy (FoDatatype *datatype)
{
  FoDatatype* length_cond;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_LENGTH_COND (datatype), NULL);

  length_cond = fo_length_cond_new ();
  fo_length_cond_set_length (length_cond,
			     FO_LENGTH_COND (datatype)->length);
  fo_length_cond_set_condity (length_cond,
			      FO_LENGTH_COND (datatype)->condity);

  return (length_cond);
}

/**
 * fo_length_cond_get_length_cond_zero:
 * 
 * Get an #FoLengthCond with the well-known value of 0pt/discard.
 * 
 * Return value: The #FoLengthCond.
 **/
FoDatatype *
fo_length_cond_get_length_cond_zero (void)
{
  static FoDatatype *length_cond = NULL;

  if (length_cond == NULL)
    {
      length_cond = fo_length_cond_new ();
    }

  return length_cond;
}
