/* Fo
 * fo-footnote-body.c: Footnote Body formatting object
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-footnote-body.c,v 1.2 2003/03/19 21:11:05 tonygraham Exp $
 *
 * See Copyright for the status of this software.
 */

#include "libfo/fo-utils.h"
#include "fo/fo-fo.h"
#include "fo/fo-fo-private.h"
#include "fo-footnote-body.h"
#include "fo-footnote-body-private.h"

enum {
  PROP_0,
  PROP_ROLE,
  PROP_SOURCE_DOCUMENT
};

static void fo_footnote_body_init        (FoFootnoteBody      *fo_footnote_body);
static void fo_footnote_body_class_init  (FoFootnoteBodyClass *klass);
static void fo_footnote_body_set_property (GObject         *object,
                                  guint            prop_id,
                                  const GValue    *value,
                                  GParamSpec      *pspec);
static void fo_footnote_body_get_property   (GObject         *object,
                                       guint            prop_id,
                                       GValue          *value,
                                       GParamSpec      *pspec);
static void fo_footnote_body_finalize    (GObject           *object);
static void fo_footnote_body_debug_dump_properties (FoFo *fo, gint depth);

static gpointer parent_class;

GType
fo_footnote_body_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoFootnoteBodyClass),
        (GBaseInitFunc) NULL,
        (GBaseFinalizeFunc) NULL,
        (GClassInitFunc) fo_footnote_body_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoFootnoteBody),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_footnote_body_init,
      };
      
      object_type = g_type_register_static (FO_TYPE_FO,
                                            "FoFootnoteBody",
                                            &object_info, 0);
    }
  
  return object_type;
}

static void
fo_footnote_body_init (FoFootnoteBody *fo_footnote_body)
{
}

static void
fo_footnote_body_class_init (FoFootnoteBodyClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  
  parent_class = g_type_class_peek_parent (klass);
  
  object_class->finalize = fo_footnote_body_finalize;

  object_class->set_property = fo_footnote_body_set_property;
  object_class->get_property = fo_footnote_body_get_property;

  FO_FO_CLASS (klass)->debug_dump_properties = fo_footnote_body_debug_dump_properties;
  FO_FO_CLASS (klass)->update_from_context = fo_footnote_body_update_from_context;

  g_object_class_install_property
    (object_class,
     PROP_ROLE,
     g_param_spec_object ("role",
			  _("Role"),
			  _("Role property"),
			  FO_TYPE_PROPERTY,
			  G_PARAM_READABLE));
  g_object_class_install_property
    (object_class,
     PROP_SOURCE_DOCUMENT,
     g_param_spec_object ("source-document",
			  _("Source Document"),
			  _("Source Document property"),
			  FO_TYPE_PROPERTY,
			  G_PARAM_READABLE));
}

static void
fo_footnote_body_finalize (GObject *object)
{
  FoFootnoteBody *fo_footnote_body;

  fo_footnote_body = FO_FOOTNOTE_BODY (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


static void
fo_footnote_body_set_property (GObject         *object,
                         guint            prop_id,
                         const GValue    *value,
                         GParamSpec      *pspec)
{
  FoFootnoteBody *fo_footnote_body;

  fo_footnote_body = FO_FOOTNOTE_BODY (object);

  switch (prop_id)
    {
    case PROP_ROLE:
      fo_footnote_body_set_role (fo_footnote_body, FO_PROPERTY_ROLE (g_value_get_object (value)));
      break;
    case PROP_SOURCE_DOCUMENT:
      fo_footnote_body_set_source_document (fo_footnote_body, FO_PROPERTY_SOURCE_DOCUMENT (g_value_get_object (value)));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
fo_footnote_body_get_property (GObject         *object,
                         guint            prop_id,
                         GValue          *value,
                         GParamSpec      *pspec)
{
  FoFootnoteBody *fo_footnote_body;

  fo_footnote_body = FO_FOOTNOTE_BODY (object);

  switch (prop_id)
    {
    case PROP_ROLE:
      g_value_set_object (value, G_OBJECT (fo_footnote_body_get_role (fo_footnote_body)));
      break;
    case PROP_SOURCE_DOCUMENT:
      g_value_set_object (value, G_OBJECT (fo_footnote_body_get_source_document (fo_footnote_body)));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/**
 * fo_footnote_body_new:
 * 
 * Creates a new #FoFootnoteBody initialized to default value.
 * 
 * Return value: the new #FoFootnoteBody
 **/
FoFo*
fo_footnote_body_new (void)
{
  return FO_FO (g_object_new (fo_footnote_body_get_type (), NULL));
}

void
fo_footnote_body_update_from_context (FoFo *fo, FoContext *context)
{
  g_return_if_fail (fo != NULL);
  g_return_if_fail (FO_IS_FOOTNOTE_BODY (fo));

  fo_footnote_body_set_role (FO_FOOTNOTE_BODY (fo),
			  fo_context_get_role (context));
  fo_footnote_body_set_source_document (FO_FOOTNOTE_BODY (fo),
			  fo_context_get_source_document (context));
}

void
fo_footnote_body_debug_dump_properties (FoFo *fo, gint depth)
{
  FoFootnoteBody *fo_footnote_body;

  g_return_if_fail (fo != NULL);
  g_return_if_fail (FO_IS_FOOTNOTE_BODY (fo));

  fo_footnote_body = FO_FOOTNOTE_BODY (fo);

  fo_object_debug_dump (fo_footnote_body->role, depth);
  fo_object_debug_dump (fo_footnote_body->source_document, depth);

  FO_FO_CLASS (parent_class)->debug_dump_properties (fo, depth + 1);
}

/**
 * fo_footnote_body_set_role:
 * @fo_footnote_body: The @FoFootnoteBody object
 * @new_role: The new "role" property value
 * 
 * Sets the #role property of @footnote_body to @new_role
 **/
void
fo_footnote_body_set_role (FoFootnoteBody *fo_footnote_body,
		         FoPropertyRole *new_role)
{
  g_return_if_fail (fo_footnote_body != NULL);
  g_return_if_fail (FO_IS_FOOTNOTE_BODY (fo_footnote_body));

  if (new_role)
    g_object_ref (G_OBJECT (new_role));
  if (fo_footnote_body->role)
    g_object_unref (G_OBJECT (fo_footnote_body->role));
  fo_footnote_body->role = FO_PROPERTY_ROLE (new_role);
  /*g_object_notify (G_OBJECT (fo_footnote_body), "role");*/
}

/**
 * fo_footnote_body_get_role:
 * @fo_footnote_body: The @FoFootnoteBody object
 *
 * Gets the #role property of @footnote_body
 *
 * Return value: The "role" property value
**/
FoPropertyRole*
fo_footnote_body_get_role (FoFootnoteBody *fo_footnote_body)
{
  g_return_val_if_fail (fo_footnote_body != NULL, NULL);
  g_return_val_if_fail (FO_IS_FOOTNOTE_BODY (fo_footnote_body), NULL);

  return fo_footnote_body->role;
}

/**
 * fo_footnote_body_set_source_document:
 * @fo_footnote_body: The @FoFootnoteBody object
 * @new_source_document: The new "source-document" property value
 * 
 * Sets the #source-document property of @footnote_body to @new_source_document
 **/
void
fo_footnote_body_set_source_document (FoFootnoteBody *fo_footnote_body,
		         FoPropertySourceDocument *new_source_document)
{
  g_return_if_fail (fo_footnote_body != NULL);
  g_return_if_fail (FO_IS_FOOTNOTE_BODY (fo_footnote_body));

  if (new_source_document)
    g_object_ref (G_OBJECT (new_source_document));
  if (fo_footnote_body->source_document)
    g_object_unref (G_OBJECT (fo_footnote_body->source_document));
  fo_footnote_body->source_document = FO_PROPERTY_SOURCE_DOCUMENT (new_source_document);
  /*g_object_notify (G_OBJECT (fo_footnote_body), "source-document");*/
}

/**
 * fo_footnote_body_get_source_document:
 * @fo_footnote_body: The @FoFootnoteBody object
 *
 * Gets the #source-document property of @footnote_body
 *
 * Return value: The "source-document" property value
**/
FoPropertySourceDocument*
fo_footnote_body_get_source_document (FoFootnoteBody *fo_footnote_body)
{
  g_return_val_if_fail (fo_footnote_body != NULL, NULL);
  g_return_val_if_fail (FO_IS_FOOTNOTE_BODY (fo_footnote_body), NULL);

  return fo_footnote_body->source_document;
}
