/* Fo
 * fo-property-dominant-baseline.c: 'dominant-baseline' property
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-property-dominant-baseline.c,v 1.4 2003/04/14 11:17:37 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include <string.h>
#include <pango/pango-xsl-attributes.h>
#include "libfo/fo-utils.h"
#include "fo-context.h"
#include "datatype/fo-datatype.h"
#include "property/fo-property-private.h"
#include "property/fo-property-text-property.h"
#include "property/fo-property-font-size.h"
#include "property/fo-property-font-style.h"
#include "property/fo-property-dominant-baseline.h"

/* Inherited: FALSE */
/* Shorthand: FALSE */
/* dominant-baseline */
/* auto | use-script | no-change | reset-size | ideographic | alphabetic | hanging | mathematical | central | middle | text-after-edge | text-before-edge | inherit */
GType
fo_enum_dominant_baseline_get_type (void)
{
  static GType etype = 0;
  if (etype == 0)
    {
      static const GEnumValue values[] = {
        { FO_ENUM_DOMINANT_BASELINE_AUTO, "FO_ENUM_DOMINANT_BASELINE_AUTO", "auto" },
        { FO_ENUM_DOMINANT_BASELINE_USE_SCRIPT, "FO_ENUM_DOMINANT_BASELINE_USE_SCRIPT", "use-script" },
        { FO_ENUM_DOMINANT_BASELINE_NO_CHANGE, "FO_ENUM_DOMINANT_BASELINE_NO_CHANGE", "no-change" },
        { FO_ENUM_DOMINANT_BASELINE_RESET_SIZE, "FO_ENUM_DOMINANT_BASELINE_RESET_SIZE", "reset-size" },
        { FO_ENUM_DOMINANT_BASELINE_IDEOGRAPHIC, "FO_ENUM_DOMINANT_BASELINE_IDEOGRAPHIC", "ideographic" },
        { FO_ENUM_DOMINANT_BASELINE_ALPHABETIC, "FO_ENUM_DOMINANT_BASELINE_ALPHABETIC", "alphabetic" },
        { FO_ENUM_DOMINANT_BASELINE_HANGING, "FO_ENUM_DOMINANT_BASELINE_HANGING", "hanging" },
        { FO_ENUM_DOMINANT_BASELINE_MATHEMATICAL, "FO_ENUM_DOMINANT_BASELINE_MATHEMATICAL", "mathematical" },
        { FO_ENUM_DOMINANT_BASELINE_CENTRAL, "FO_ENUM_DOMINANT_BASELINE_CENTRAL", "central" },
        { FO_ENUM_DOMINANT_BASELINE_MIDDLE, "FO_ENUM_DOMINANT_BASELINE_MIDDLE", "middle" },
        { FO_ENUM_DOMINANT_BASELINE_TEXT_AFTER_EDGE, "FO_ENUM_DOMINANT_BASELINE_TEXT_AFTER_EDGE", "text-after-edge" },
        { FO_ENUM_DOMINANT_BASELINE_TEXT_BEFORE_EDGE, "FO_ENUM_DOMINANT_BASELINE_TEXT_BEFORE_EDGE", "text-before-edge" },
        { 0, NULL, NULL }
      };
      etype = g_enum_register_static ("FoEnumDominantBaseline", values);
    }
  return etype;
}

struct _FoPropertyDominantBaseline
{
  FoProperty parent_instance;
};

struct _FoPropertyDominantBaselineClass
{
  FoPropertyClass parent_class;
};

static void fo_property_dominant_baseline_init         (FoPropertyDominantBaseline      *property_dominant_baseline);
static void fo_property_dominant_baseline_base_init   (FoPropertyDominantBaselineClass *klass);
static void fo_property_dominant_baseline_base_finalize   (FoPropertyDominantBaselineClass *klass);
static void fo_property_dominant_baseline_class_init   (FoPropertyDominantBaselineClass *klass);
static void fo_property_dominant_baseline_finalize     (GObject       *object);
static void fo_property_dominant_baseline_text_property_init (FoPropertyTextPropertyIface *iface);

static FoDatatype* fo_property_dominant_baseline_resolve_enum (const gchar *token,
                                                               FoContext   *context,
                                                               GError     **error);
static FoDatatype* fo_property_dominant_baseline_validate (FoDatatype *datatype,
                                                           FoContext  *context,
                                                           GError    **error);
PangoAttribute * fo_property_dominant_baseline_new_attr_from_context (FoProperty *property,
								      FoContext  *context);

static const gchar class_name[] = "dominant-baseline";
static gpointer parent_class;
static FoDatatype *enum_auto;
static FoDatatype *enum_use_script;
static FoDatatype *enum_no_change;
static FoDatatype *enum_reset_size;
static FoDatatype *enum_ideographic;
static FoDatatype *enum_alphabetic;
static FoDatatype *enum_hanging;
static FoDatatype *enum_mathematical;
static FoDatatype *enum_central;
static FoDatatype *enum_middle;
static FoDatatype *enum_text_after_edge;
static FoDatatype *enum_text_before_edge;

/**
 * fo_property_dominant_baseline_get_type:
 * 
 * Register the #FoPropertyDominantBaseline type if not already registered and
 * return its #GType value.
 * 
 * Return value: #GType of #FoPropertyDominantBaseline.
 **/
GType
fo_property_dominant_baseline_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoPropertyDominantBaselineClass),
        (GBaseInitFunc) fo_property_dominant_baseline_base_init,
        (GBaseFinalizeFunc) fo_property_dominant_baseline_base_finalize,
        (GClassInitFunc) fo_property_dominant_baseline_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoPropertyDominantBaseline),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_property_dominant_baseline_init,
	NULL		/* value_table */
      };

      static const GInterfaceInfo fo_property_text_property_info =
      {
	(GInterfaceInitFunc) fo_property_dominant_baseline_text_property_init, /* interface_init */
        NULL,
        NULL
      };

      object_type = g_type_register_static (FO_TYPE_PROPERTY,
                                            class_name,
                                            &object_info, 0);
      g_type_add_interface_static (object_type,
                                   FO_TYPE_PROPERTY_TEXT_PROPERTY,
                                   &fo_property_text_property_info);
    }

  return object_type;
}

/**
 * fo_property_dominant_baseline_init:
 * @dominant_baseline: #FoPropertyDominantBaseline object to initialise.
 * 
 * Implements #GInstanceInitFunc for #FoPropertyDominantBaseline.
 **/
void
fo_property_dominant_baseline_init (FoPropertyDominantBaseline *dominant_baseline)
{
  FO_PROPERTY (dominant_baseline)->value =
    g_object_ref (enum_auto);
}

/**
 * fo_property_dominant_baseline_base_init:
 * @klass: #FoPropertyDominantBaselineClass object to initialise.
 * 
 * Implements #GBaseInitFunc for #FoPropertyDominantBaselineClass.
 **/
void
fo_property_dominant_baseline_base_init (FoPropertyDominantBaselineClass *klass)
{
  enum_auto =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_DOMINANT_BASELINE),
                  "value",
                  FO_ENUM_DOMINANT_BASELINE_AUTO,
                  NULL);
  enum_use_script =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_DOMINANT_BASELINE),
                  "value",
                  FO_ENUM_DOMINANT_BASELINE_USE_SCRIPT,
                  NULL);
  enum_no_change =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_DOMINANT_BASELINE),
                  "value",
                  FO_ENUM_DOMINANT_BASELINE_NO_CHANGE,
                  NULL);
  enum_reset_size =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_DOMINANT_BASELINE),
                  "value",
                  FO_ENUM_DOMINANT_BASELINE_RESET_SIZE,
                  NULL);
  enum_ideographic =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_DOMINANT_BASELINE),
                  "value",
                  FO_ENUM_DOMINANT_BASELINE_IDEOGRAPHIC,
                  NULL);
  enum_alphabetic =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_DOMINANT_BASELINE),
                  "value",
                  FO_ENUM_DOMINANT_BASELINE_ALPHABETIC,
                  NULL);
  enum_hanging =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_DOMINANT_BASELINE),
                  "value",
                  FO_ENUM_DOMINANT_BASELINE_HANGING,
                  NULL);
  enum_mathematical =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_DOMINANT_BASELINE),
                  "value",
                  FO_ENUM_DOMINANT_BASELINE_MATHEMATICAL,
                  NULL);
  enum_central =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_DOMINANT_BASELINE),
                  "value",
                  FO_ENUM_DOMINANT_BASELINE_CENTRAL,
                  NULL);
  enum_middle =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_DOMINANT_BASELINE),
                  "value",
                  FO_ENUM_DOMINANT_BASELINE_MIDDLE,
                  NULL);
  enum_text_after_edge =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_DOMINANT_BASELINE),
                  "value",
                  FO_ENUM_DOMINANT_BASELINE_TEXT_AFTER_EDGE,
                  NULL);
  enum_text_before_edge =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_DOMINANT_BASELINE),
                  "value",
                  FO_ENUM_DOMINANT_BASELINE_TEXT_BEFORE_EDGE,
                  NULL);
}

/**
 * fo_property_dominant_baseline_base_finalize:
 * @klass: #FoPropertyDominantBaselineClass object to finalise.
 * 
 * Implements #GBaseFinalizeFunc for #FoPropertyDominantBaselineClass.
 **/
void
fo_property_dominant_baseline_base_finalize (FoPropertyDominantBaselineClass *klass)
{
  g_object_unref (enum_auto);
  g_object_unref (enum_use_script);
  g_object_unref (enum_no_change);
  g_object_unref (enum_reset_size);
  g_object_unref (enum_ideographic);
  g_object_unref (enum_alphabetic);
  g_object_unref (enum_hanging);
  g_object_unref (enum_mathematical);
  g_object_unref (enum_central);
  g_object_unref (enum_middle);
  g_object_unref (enum_text_after_edge);
  g_object_unref (enum_text_before_edge);
}

/**
 * fo_property_dominant_baseline_class_init:
 * @klass: #FoPropertyDominantBaselineClass object to initialise.
 * 
 * Implements #GClassInitFunc for #FoPropertyDominantBaselineClass.
 **/
void
fo_property_dominant_baseline_class_init (FoPropertyDominantBaselineClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  FoPropertyClass *property_class = FO_PROPERTY_CLASS (klass);

  parent_class = g_type_class_peek_parent (klass);

  object_class->finalize = fo_property_dominant_baseline_finalize;

  property_class->is_inherited = FALSE;
  property_class->is_shorthand = FALSE;
  property_class->resolve_enum =
    fo_property_dominant_baseline_resolve_enum;
  property_class->validate =
    fo_property_dominant_baseline_validate;
  property_class->get_initial =
    fo_property_dominant_baseline_get_initial;
}

/**
 * fo_property_dominant_baseline_finalize:
 * @object: #FoPropertyDominantBaseline object to finalize.
 * 
 * Implements #GObjectFinalizeFunc for #FoPropertyDominantBaseline.
 **/
void
fo_property_dominant_baseline_finalize (GObject *object)
{
  FoPropertyDominantBaseline *dominant_baseline;

  dominant_baseline = FO_PROPERTY_DOMINANT_BASELINE (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}

/**
 * fo_property_dominant_baseline_text_property_init:
 * @iface: #FoPropertyTextPropertyIFace structure for this class
 * 
 * Initialize #FoPropertyTextPropertyIface interface for this class
 **/
void
fo_property_dominant_baseline_text_property_init (FoPropertyTextPropertyIface *iface)
{
  iface->new_attr_from_context = fo_property_dominant_baseline_new_attr_from_context;
}

/**
 * fo_property_dominant_baseline_new:
 * 
 * Creates a new #FoPropertyDominantBaseline initialized to default value.
 * 
 * Return value: the new #FoPropertyDominantBaseline.
 **/
FoProperty*
fo_property_dominant_baseline_new (void)
{
  FoProperty* dominant_baseline;

  dominant_baseline =
    FO_PROPERTY (g_object_new (fo_property_dominant_baseline_get_type (),
                               NULL));

  return dominant_baseline;
}

/* FIXME: Why is this necessary?  This should happen in fo_context_merge() */
void
fo_property_dominant_baseline_resolve (FoContext *current_context,
				       FoContext *parent_context)
{
  g_return_if_fail (current_context != NULL);
  g_return_if_fail (FO_IS_CONTEXT (current_context));
  g_return_if_fail (parent_context != NULL);
  g_return_if_fail (FO_IS_CONTEXT (parent_context));

  if (fo_context_get_dominant_baseline (current_context) == NULL)
    {
      fo_context_set_dominant_baseline (current_context,
					fo_property_dominant_baseline_get_initial ());
    }
}

/**
 * fo_property_dominant_baseline_resolve_enum:
 * @token:   Token from the XML attribute value to be evaluated as an
 *           enumeration token.
 * @context: #FoContext object from which to possibly inherit values.
 * @error:   Information about any error that has occurred.
 * 
 * Compare @token against the enumeration tokens that are valid for the
 * current FO property.  If @token is valid, returns either an #FoEnum datatype
 * representing the enumeration token or a different datatype representing
 * the enumeration token's resolved value.  If @token is not valid,
 * sets @error and returns NULL.
 * 
 * Return value: Resolved enumeration value or NULL.
 **/
FoDatatype*
fo_property_dominant_baseline_resolve_enum (const gchar *token,
                                            FoContext   *context,
                                            GError     **error)
{
  g_return_val_if_fail (token != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (strcmp (token, "auto") == 0)
    {
      return g_object_ref (enum_auto);
    }
  else if (strcmp (token, "use-script") == 0)
    {
      return g_object_ref (enum_use_script);
    }
  else if (strcmp (token, "no-change") == 0)
    {
      return g_object_ref (enum_no_change);
    }
  else if (strcmp (token, "reset-size") == 0)
    {
      return g_object_ref (enum_reset_size);
    }
  else if (strcmp (token, "ideographic") == 0)
    {
      return g_object_ref (enum_ideographic);
    }
  else if (strcmp (token, "alphabetic") == 0)
    {
      return g_object_ref (enum_alphabetic);
    }
  else if (strcmp (token, "hanging") == 0)
    {
      return g_object_ref (enum_hanging);
    }
  else if (strcmp (token, "mathematical") == 0)
    {
      return g_object_ref (enum_mathematical);
    }
  else if (strcmp (token, "central") == 0)
    {
      return g_object_ref (enum_central);
    }
  else if (strcmp (token, "middle") == 0)
    {
      return g_object_ref (enum_middle);
    }
  else if (strcmp (token, "text-after-edge") == 0)
    {
      return g_object_ref (enum_text_after_edge);
    }
  else if (strcmp (token, "text-before-edge") == 0)
    {
      return g_object_ref (enum_text_before_edge);
    }
  else
    {
      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_ENUMERATION_TOKEN,
		   _(fo_fo_error_messages[FO_FO_ERROR_ENUMERATION_TOKEN]),
		   class_name,
		   token);
      return NULL;
    }
}

/**
 * fo_property_dominant_baseline_validate:
 * @datatype: #FoDatatype to be validated against allowed datatypes and
 *            values for current property.
 * @context:  #FoContext object from which to possibly inherit values.
 * @error:    Information about any error that has occurred.
 * 
 * Validates @datatype against allowed values.  Returns @datatype, a
 * replacement datatype value, or NULL if validation failed.
 * 
 * Return value: Valid datatype value or NULL.
 **/
FoDatatype*
fo_property_dominant_baseline_validate (FoDatatype *datatype,
                                        FoContext  *context,
                                        GError    **error)
{
  FoDatatype *new_datatype;
  GError     *tmp_error = NULL;
  gchar      *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (FO_IS_ENUM (datatype))
    {
      if ((G_ENUM_CLASS_TYPE (fo_enum_get_enum_class (datatype))
           == FO_TYPE_ENUM_DOMINANT_BASELINE) &&
	  (g_enum_get_value (fo_enum_get_enum_class (datatype),
			     fo_enum_get_value (datatype)) != NULL))
	{
	  return datatype;
	}
      else
	{
	  gchar *datatype_sprintf = fo_object_sprintf (datatype);

	  g_set_error (error,
		       FO_FO_ERROR,
		       FO_FO_ERROR_DATATYPE,
		       _(fo_fo_error_messages[FO_FO_ERROR_DATATYPE]),
		       class_name,
		       datatype_sprintf,
		       g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

	  g_object_unref (datatype);

	  g_free (datatype_sprintf);

	  return NULL;
	}
    }
  else if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype =
        fo_property_dominant_baseline_resolve_enum (token, context, &tmp_error);

      g_object_unref (datatype);

      if (tmp_error != NULL)
	{
	  g_propagate_error (error, tmp_error);
	  return NULL;
	}

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype =
        fo_property_dominant_baseline_resolve_enum (token, context, &tmp_error);

      g_object_unref (datatype);

      if (tmp_error != NULL)
	{
	  g_propagate_error (error, tmp_error);
	  return NULL;
	}

      return new_datatype;
    }
  else
    {
      gchar *datatype_sprintf = fo_object_sprintf (datatype);

      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_DATATYPE,
		   _(fo_fo_error_messages[FO_FO_ERROR_DATATYPE]),
		   class_name,
		   datatype_sprintf,
		   g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      g_free (datatype_sprintf);

      return NULL;
    }
}

/**
 * fo_property_dominant_baseline_get_initial:
 * 
 * Get an instance of the property with the correct initial value.
 * 
 * Return value: An instance of the property.
 **/
FoProperty*
fo_property_dominant_baseline_get_initial (void)
{
  static FoProperty *dominant_baseline = NULL;

  if (dominant_baseline == NULL)
    {
      dominant_baseline = fo_property_dominant_baseline_new ();
    }

  return dominant_baseline;
}

/**
 * fo_property_dominant_baseline_to_pango_baseline:
 * @baseline: #FoEnumDominantBaseline to convert to #PangoBaseline value.
 * 
 * Gets the #PangoBaseline value corresponding to the value of
 * @baseline.
 * 
 * Return value: A #PangoBaseline value, which could be
 * #PANGO_BASELINE_INVALID.
 **/
static PangoBaseline
fo_property_dominant_baseline_to_pango_baseline (FoEnumDominantBaseline baseline)
{
  switch (baseline)
    {
    case FO_ENUM_DOMINANT_BASELINE_AUTO:
      return PANGO_BASELINE_AUTO;
    case FO_ENUM_DOMINANT_BASELINE_RESET_SIZE:
      return PANGO_BASELINE_RESET_SIZE;
    case FO_ENUM_DOMINANT_BASELINE_USE_SCRIPT:
      return PANGO_BASELINE_USE_SCRIPT;
    case FO_ENUM_DOMINANT_BASELINE_IDEOGRAPHIC:
      return PANGO_BASELINE_IDEOGRAPHIC;
    case FO_ENUM_DOMINANT_BASELINE_ALPHABETIC:
      return PANGO_BASELINE_ALPHABETIC;
    case FO_ENUM_DOMINANT_BASELINE_HANGING:
      return PANGO_BASELINE_HANGING;
    case FO_ENUM_DOMINANT_BASELINE_MATHEMATICAL:
      return PANGO_BASELINE_MATHEMATICAL;
    case FO_ENUM_DOMINANT_BASELINE_CENTRAL:
      return PANGO_BASELINE_CENTRAL;
    case FO_ENUM_DOMINANT_BASELINE_MIDDLE:
      return PANGO_BASELINE_MIDDLE;
    case FO_ENUM_DOMINANT_BASELINE_TEXT_BEFORE_EDGE:
      return PANGO_BASELINE_TEXT_BEFORE_EDGE;
    case FO_ENUM_DOMINANT_BASELINE_TEXT_AFTER_EDGE:
      return PANGO_BASELINE_TEXT_AFTER_EDGE;
    default:
      return PANGO_BASELINE_INVALID;
    }
}

static PangoStyle
fo_font_style_to_pango_style (gint font_style)
{
  gint pango_font_style = PANGO_STYLE_NORMAL;

  /* Pango doesn't support 'backslant' styles */
  if (font_style == FO_ENUM_FONT_STYLE_ITALIC)
    {
      pango_font_style = PANGO_STYLE_ITALIC;
    }
  else if (font_style == FO_ENUM_FONT_STYLE_OBLIQUE)
    {
      pango_font_style = PANGO_STYLE_OBLIQUE;
    }

  return pango_font_style;
}

/**
 * fo_property_dominant_baseline_new_attr:
 * @property: #FoProperty from which to create a #PangoAttribute.
 * @context:  #FoContext from which to take additional property values.
 * 
 * Makes a new #PangoAttribute representing the value of @property.
 * 
 * Return value: New #PangoAttribute for @property.
 **/
PangoAttribute*
fo_property_dominant_baseline_new_attr_from_context (FoProperty *property,
						     FoContext  *context)
{
  PangoFontDescription *font_desc;
  guint dominant_baseline;

  g_return_val_if_fail (FO_IS_PROPERTY_DOMINANT_BASELINE (property), NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);

  dominant_baseline =
    fo_enum_get_value (fo_property_get_value (property));

  if (dominant_baseline != FO_ENUM_DOMINANT_BASELINE_RESET_SIZE)
    {
      PangoBaseline pango_dominant_baseline =
	fo_property_dominant_baseline_to_pango_baseline (dominant_baseline);
  
      font_desc = pango_font_description_new ();

      pango_font_description_set_family (font_desc,
					 fo_string_get_value (fo_property_get_value (fo_context_get_font_family (context))));
      pango_font_description_set_size (font_desc,
				       fo_length_get_value (fo_property_get_value (fo_context_get_font_size (context))) * PANGO_SCALE);
      pango_font_description_set_weight (font_desc,
					 fo_integer_get_value (fo_property_get_value (fo_context_get_font_weight (context))));
      pango_font_description_set_style (font_desc,
					fo_font_style_to_pango_style (fo_enum_get_value (fo_property_get_value (fo_context_get_font_style (context)))));
      pango_font_description_set_stretch (font_desc,
					  fo_enum_get_value (fo_property_get_value (fo_context_get_font_stretch (context))));
      pango_font_description_set_variant (font_desc,
					  fo_enum_get_value (fo_property_get_value (fo_context_get_font_variant (context))));

      return pango_attr_dominant_baseline_new (pango_dominant_baseline,
					       font_desc);
    }
  else
    {
      return pango_attr_db_reset_size_new (TRUE);
    }
}      
