/* Fo
 * fo-property-letter-value.c: letter-value property
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-property-letter-value.c,v 1.2 2003/03/21 14:41:36 tonygraham Exp $
 *
 * See Copyright for the status of this software.
 */

#include <string.h>
#include "libfo/fo-utils.h"
#include "fo-datatype.h"
#include "fo-property-private.h"
#include "fo-property-font-size.h"
#include <fo-property-letter-value.h>

/* letter-value */
/* auto | alphabetic | traditional */
GType
fo_enum_letter_value_get_type (void)
{
  static GType etype = 0;
  if (etype == 0)
    {
      static const GEnumValue values[] = {
        { FO_ENUM_LETTER_VALUE_AUTO, "FO_ENUM_LETTER_VALUE_AUTO", "auto" },
        { FO_ENUM_LETTER_VALUE_ALPHABETIC, "FO_ENUM_LETTER_VALUE_ALPHABETIC", "alphabetic" },
        { FO_ENUM_LETTER_VALUE_TRADITIONAL, "FO_ENUM_LETTER_VALUE_TRADITIONAL", "traditional" },
        { 0, NULL, NULL }
      };
      etype = g_enum_register_static ("FoEnumLetterValue", values);
    }
  return etype;
}

enum {
  PROP_0,
  PROP_VALUE
};

struct _FoPropertyLetterValue
{
  FoProperty parent_instance;

  FoDatatype *value;
};

struct _FoPropertyLetterValueClass
{
  FoPropertyClass parent_class;
  
};

static void fo_property_letter_value_init         (FoPropertyLetterValue      *property_letter_value);
static void fo_property_letter_value_base_class_init   (FoPropertyLetterValueClass *klass);
static void fo_property_letter_value_base_class_finalize   (FoPropertyLetterValueClass *klass);
static void fo_property_letter_value_class_init   (FoPropertyLetterValueClass *klass);
static void fo_property_letter_value_set_property (GObject       *object,
				    guint          prop_id,
				    const GValue  *value,
				    GParamSpec    *pspec);
static void fo_property_letter_value_get_property (GObject       *object,
				    guint          prop_id,
				    GValue        *value,
				    GParamSpec    *pspec);
static void fo_property_letter_value_finalize     (GObject       *object);

FoDatatype* fo_property_letter_value_copy(FoDatatype *datatype);
static FoDatatype* fo_property_letter_value_resolve_enum (const gchar *token,
							 FoContext *context, GError **err);
static FoDatatype* fo_property_letter_value_validate (FoDatatype *datatype,
						     FoContext *context);
static gchar*      fo_property_letter_value_sprintf (FoObject *object);

static gpointer parent_class;
static FoDatatype *enum_auto;
static FoDatatype *enum_alphabetic;
static FoDatatype *enum_traditional;

GType
fo_property_letter_value_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoPropertyLetterValueClass),
        (GBaseInitFunc) fo_property_letter_value_base_class_init,
        (GBaseFinalizeFunc) fo_property_letter_value_base_class_finalize,
        (GClassInitFunc) fo_property_letter_value_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoPropertyLetterValue),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_property_letter_value_init,
      };
      
      object_type = g_type_register_static (FO_TYPE_PROPERTY,
                                            "FoPropertyLetterValue",
                                            &object_info, 0);
    }
  
  return object_type;
}

static void
fo_property_letter_value_init (FoPropertyLetterValue *letter_value)
{
  letter_value->value = NULL;
}

static void
fo_property_letter_value_base_class_init (FoPropertyLetterValueClass *klass)
{
  enum_auto =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_LETTER_VALUE),
                  "value",
                  FO_ENUM_LETTER_VALUE_AUTO,
                  NULL);
  enum_alphabetic =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_LETTER_VALUE),
                  "value",
                  FO_ENUM_LETTER_VALUE_ALPHABETIC,
                  NULL);
  enum_traditional =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_LETTER_VALUE),
                  "value",
                  FO_ENUM_LETTER_VALUE_TRADITIONAL,
                  NULL);
}

static void
fo_property_letter_value_base_class_finalize (FoPropertyLetterValueClass *klass)
{  g_object_unref (enum_auto);
  g_object_unref (enum_alphabetic);
  g_object_unref (enum_traditional);
}

static void
fo_property_letter_value_class_init (FoPropertyLetterValueClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  
  parent_class = g_type_class_peek_parent (klass);
  
  object_class->finalize = fo_property_letter_value_finalize;

  object_class->set_property = fo_property_letter_value_set_property;
  object_class->get_property = fo_property_letter_value_get_property;

  g_object_class_install_property (object_class,
                                   PROP_VALUE,
                                   g_param_spec_object ("value",
						     _("letter-value value"),
						     _("letter-value property value"),
							FO_TYPE_DATATYPE,
						     G_PARAM_READWRITE));

  FO_PROPERTY_CLASS(klass)->is_inherited = FALSE;
  FO_PROPERTY_CLASS(klass)->new_from_expression =
    fo_property_letter_value_new_from_expression;
  FO_OBJECT_CLASS(klass)->sprintf =
    fo_property_letter_value_sprintf;
  FO_PROPERTY_CLASS(klass)->set_value =
    fo_property_letter_value_set_value;
  FO_PROPERTY_CLASS(klass)->get_value =
    fo_property_letter_value_get_value;
}

static void
fo_property_letter_value_finalize (GObject *object)
{
  FoPropertyLetterValue *letter_value;

  letter_value = FO_PROPERTY_LETTER_VALUE (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


static void
fo_property_letter_value_set_property (GObject         *object,
                         guint            prop_id,
                         const GValue    *value,
                         GParamSpec      *pspec)
{
  FoPropertyLetterValue *letter_value;

  letter_value = FO_PROPERTY_LETTER_VALUE (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      fo_property_letter_value_set_value (FO_PROPERTY (letter_value), g_value_get_object (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
fo_property_letter_value_get_property (GObject         *object,
                         guint            prop_id,
                         GValue          *value,
                         GParamSpec      *pspec)
{
  FoPropertyLetterValue *letter_value;

  letter_value = FO_PROPERTY_LETTER_VALUE (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      g_value_set_instance (value,
			  fo_property_letter_value_get_value (FO_PROPERTY (letter_value)));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/**
 * fo_property_letter_value_new:
 * 
 * Creates a new #FoPropertyLetterValue initialized to default value.
 * 
 * Return value: the new #FoPropertyLetterValue
 **/
FoProperty*
fo_property_letter_value_new (void)
{
  FoProperty* letter_value;

  letter_value =
    FO_PROPERTY (g_object_new (fo_property_letter_value_get_type (), NULL));
  
  return letter_value;
}

FoDatatype*
fo_property_letter_value_resolve_enum (const gchar *token, FoContext *context, GError **err)
{
  g_return_val_if_fail (token != NULL, NULL);

  if (strcmp (token, "auto") == 0)
    {
      return g_object_ref (enum_auto);
    }
  else if (strcmp (token, "alphabetic") == 0)
    {
      return g_object_ref (enum_alphabetic);
    }
  else if (strcmp (token, "traditional") == 0)
    {
      return g_object_ref (enum_traditional);
    }
  else
    {
      g_warning("Unrecognized letter-value enumeration token: %s", token);
      return NULL;
    }
}

FoDatatype*
fo_property_letter_value_validate (FoDatatype *datatype, FoContext *context)
{
  FoDatatype *new_datatype;
  gchar *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);

  if (FO_IS_ENUM (datatype))
    {
      if ((G_ENUM_CLASS_TYPE (fo_enum_get_enum_class (datatype))
           == FO_TYPE_ENUM_LETTER_VALUE) &&
	  (g_enum_get_value (fo_enum_get_enum_class (datatype),
			     fo_enum_get_value (datatype)) != NULL))
	{
	  return datatype;
	}
      else
	{
	  g_object_unref (datatype);

	  return NULL;
	}
    }
  else if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype = fo_property_letter_value_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype = fo_property_letter_value_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else
    {
      g_warning("Invalid letter-value datatype: %s (%s)",
	        fo_object_sprintf (FO_OBJECT (datatype)),
		g_type_name(G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      return NULL;
    }
}

FoProperty*
fo_property_letter_value_new_from_expression (const gchar *expression,
					   FoContext *context,
					   FoProperty *current_font_size,
					   FoFo *fo_node,
					   GError **error)
{
  FoProperty *letter_value;
  FoDatatype *datatype;

  g_return_val_if_fail (expression != NULL, NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail
    (current_font_size == NULL || FO_IS_PROPERTY_FONT_SIZE (current_font_size),
     NULL);

  letter_value = fo_property_letter_value_new ();

  datatype =
    fo_property_eval_expression (expression,
				 "letter-value",
				 context,
				 fo_property_letter_value_resolve_enum,
				 current_font_size,
				 fo_node);

  datatype = fo_property_letter_value_validate (datatype, context);

  fo_property_letter_value_set_value (letter_value, datatype);

  return letter_value;
}

/* value */
void
fo_property_letter_value_set_value (FoProperty *property,
		       FoDatatype *new_value)
{
  FoPropertyLetterValue *letter_value;

  g_return_if_fail (property != NULL);

  letter_value = FO_PROPERTY_LETTER_VALUE (property);

  letter_value->value = new_value;
  /*g_object_notify(G_OBJECT(letter_value), "value");*/
}

FoDatatype*
fo_property_letter_value_get_value (FoProperty *property)
{
  FoPropertyLetterValue *letter_value;

  g_return_val_if_fail (property != NULL, NULL);

  letter_value = FO_PROPERTY_LETTER_VALUE (property);

  return letter_value->value;
}

gchar*
fo_property_letter_value_sprintf (FoObject *object)
{
  FoPropertyLetterValue *letter_value;
  FoDatatype *datatype;

  g_return_val_if_fail (object != NULL, NULL);

  letter_value = FO_PROPERTY_LETTER_VALUE (object);
  datatype = fo_property_letter_value_get_value (letter_value);

  return
    g_strdup_printf ("letter-value: %s",
		     fo_object_sprintf (FO_OBJECT (datatype)));
}

FoProperty*
fo_property_letter_value_get_initial (void)
{
  FoProperty *letter_value;

  letter_value = fo_property_letter_value_new ();

  /* Set the initial value here */
  fo_property_letter_value_set_value (letter_value,
				      enum_auto);

  return letter_value;
}
