/* Fo
 * fo-property-margin-top.c: margin-top property
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-property-margin-top.c,v 1.2 2003/03/21 14:41:38 tonygraham Exp $
 *
 * See Copyright for the status of this software.
 */

#include <string.h>
#include "libfo/fo-utils.h"
#include "fo-datatype.h"
#include "fo-property-private.h"
#include "fo-property-font-size.h"
#include <fo-property-margin-private.h>
#include <fo-property-margin-top.h>

/* margin-top */
/* <margin-width> | inherit */
GType
fo_enum_margin_top_get_type (void)
{
  static GType etype = 0;
  if (etype == 0)
    {
      static const GEnumValue values[] = {
        { FO_ENUM_MARGIN_TOP_AUTO, "FO_ENUM_MARGIN_TOP_AUTO", "auto" },
        { FO_ENUM_MARGIN_TOP_INHERIT, "FO_ENUM_MARGIN_TOP_INHERIT", "inherit" },
        { 0, NULL, NULL }
      };
      etype = g_enum_register_static ("FoEnumMarginTop", values);
    }
  return etype;
}

enum {
  PROP_0,
  PROP_VALUE
};

struct _FoPropertyMarginTop
{
  FoPropertyMargin parent_instance;

  FoDatatype *value;
};

struct _FoPropertyMarginTopClass
{
  FoPropertyMarginClass parent_class;
  
};

static void fo_property_margin_top_init         (FoPropertyMarginTop      *property_margin_top);
static void fo_property_margin_top_base_class_init   (FoPropertyMarginTopClass *klass);
static void fo_property_margin_top_base_class_finalize   (FoPropertyMarginTopClass *klass);
static void fo_property_margin_top_class_init   (FoPropertyMarginTopClass *klass);
static void fo_property_margin_top_set_property (GObject       *object,
				    guint          prop_id,
				    const GValue  *value,
				    GParamSpec    *pspec);
static void fo_property_margin_top_get_property (GObject       *object,
				    guint          prop_id,
				    GValue        *value,
				    GParamSpec    *pspec);
static void fo_property_margin_top_finalize     (GObject       *object);

FoDatatype* fo_property_margin_top_copy(FoDatatype *datatype);
static FoDatatype* fo_property_margin_top_resolve_enum (const gchar *token,
							 FoContext *context, GError **err);
static FoDatatype* fo_property_margin_top_validate (FoDatatype *datatype,
						     FoContext *context);
static gchar*      fo_property_margin_top_sprintf (FoObject *object);

static gpointer parent_class;
static FoDatatype *enum_auto;

GType
fo_property_margin_top_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoPropertyMarginTopClass),
        (GBaseInitFunc) fo_property_margin_top_base_class_init,
        (GBaseFinalizeFunc) fo_property_margin_top_base_class_finalize,
        (GClassInitFunc) fo_property_margin_top_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoPropertyMarginTop),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_property_margin_top_init,
      };
      
      object_type = g_type_register_static (FO_TYPE_PROPERTY_MARGIN,
                                            "FoPropertyMarginTop",
                                            &object_info, 0);
    }
  
  return object_type;
}

static void
fo_property_margin_top_init (FoPropertyMarginTop *margin_top)
{
  margin_top->value = NULL;
}

static void
fo_property_margin_top_base_class_init (FoPropertyMarginTopClass *klass)
{
  enum_auto =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_MARGIN_TOP),
                  "value",
                  FO_ENUM_MARGIN_TOP_AUTO,
                  NULL);
}

static void
fo_property_margin_top_base_class_finalize (FoPropertyMarginTopClass *klass)
{  g_object_unref (enum_auto);
}

static void
fo_property_margin_top_class_init (FoPropertyMarginTopClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  
  parent_class = g_type_class_peek_parent (klass);
  
  object_class->finalize = fo_property_margin_top_finalize;

  object_class->set_property = fo_property_margin_top_set_property;
  object_class->get_property = fo_property_margin_top_get_property;

  g_object_class_install_property (object_class,
                                   PROP_VALUE,
                                   g_param_spec_object ("value",
						     _("margin-top value"),
						     _("margin-top property value"),
							FO_TYPE_DATATYPE,
						     G_PARAM_READWRITE));

  FO_PROPERTY_CLASS(klass)->is_inherited = FALSE;
  FO_PROPERTY_CLASS(klass)->new_from_expression =
    fo_property_margin_top_new_from_expression;
  FO_OBJECT_CLASS(klass)->sprintf =
    fo_property_margin_top_sprintf;
  FO_PROPERTY_CLASS(klass)->set_value =
    fo_property_margin_top_set_value;
  FO_PROPERTY_CLASS(klass)->get_value =
    fo_property_margin_top_get_value;
}

static void
fo_property_margin_top_finalize (GObject *object)
{
  FoPropertyMarginTop *margin_top;

  margin_top = FO_PROPERTY_MARGIN_TOP (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


static void
fo_property_margin_top_set_property (GObject         *object,
                         guint            prop_id,
                         const GValue    *value,
                         GParamSpec      *pspec)
{
  FoPropertyMarginTop *margin_top;

  margin_top = FO_PROPERTY_MARGIN_TOP (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      fo_property_margin_top_set_value (FO_PROPERTY (margin_top), g_value_get_object (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
fo_property_margin_top_get_property (GObject         *object,
                         guint            prop_id,
                         GValue          *value,
                         GParamSpec      *pspec)
{
  FoPropertyMarginTop *margin_top;

  margin_top = FO_PROPERTY_MARGIN_TOP (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      g_value_set_instance (value,
			  fo_property_margin_top_get_value (FO_PROPERTY (margin_top)));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/**
 * fo_property_margin_top_new:
 * 
 * Creates a new #FoPropertyMarginTop initialized to default value.
 * 
 * Return value: the new #FoPropertyMarginTop
 **/
FoProperty*
fo_property_margin_top_new (void)
{
  FoProperty* margin_top;

  margin_top =
    FO_PROPERTY (g_object_new (fo_property_margin_top_get_type (), NULL));
  
  return margin_top;
}

FoDatatype*
fo_property_margin_top_resolve_enum (const gchar *token, FoContext *context, GError **err)
{
  g_return_val_if_fail (token != NULL, NULL);

  if (strcmp (token, "auto") == 0)
    {
      return g_object_ref (enum_auto);
    }
  else if (strcmp (token, "inherit") == 0)
    {
      return g_object_ref (fo_context_get_margin_top (context));
    }
  else
    {
      g_warning("Unrecognized margin-top enumeration token: %s", token);
      return NULL;
    }
}

FoDatatype*
fo_property_margin_top_validate (FoDatatype *datatype, FoContext *context)
{
  FoDatatype *new_datatype;
  gchar *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);

  if (FO_IS_ENUM (datatype))
    {
      if ((G_ENUM_CLASS_TYPE (fo_enum_get_enum_class (datatype))
           == FO_TYPE_ENUM_MARGIN_TOP) &&
	  (g_enum_get_value (fo_enum_get_enum_class (datatype),
			     fo_enum_get_value (datatype)) != NULL))
	{
	  return datatype;
	}
      else
	{
	  g_object_unref (datatype);

	  return NULL;
	}
    }
  else if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype = fo_property_margin_top_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype = fo_property_margin_top_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else if (FO_IS_LENGTH (datatype))
    {
      return datatype;
    }
  else if (FO_IS_PERCENTAGE (datatype))
    {
      return datatype;
    }
  else
    {
      g_warning("Invalid margin-top datatype: %s (%s)",
	        fo_object_sprintf (FO_OBJECT (datatype)),
		g_type_name(G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      return NULL;
    }
}

FoProperty*
fo_property_margin_top_new_from_expression (const gchar *expression,
					   FoContext *context,
					   FoProperty *current_font_size,
					   FoFo *fo_node,
					   GError **error)
{
  FoProperty *margin_top;
  FoDatatype *datatype;

  g_return_val_if_fail (expression != NULL, NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail
    (current_font_size == NULL || FO_IS_PROPERTY_FONT_SIZE (current_font_size),
     NULL);

  margin_top = fo_property_margin_top_new ();

  datatype =
    fo_property_eval_expression (expression,
				 "margin-top",
				 context,
				 fo_property_margin_top_resolve_enum,
				 current_font_size,
				 fo_node);

  datatype = fo_property_margin_top_validate (datatype, context);

  fo_property_margin_top_set_value (margin_top, datatype);

  return margin_top;
}

/* value */
void
fo_property_margin_top_set_value (FoProperty *property,
		       FoDatatype *new_value)
{
  FoPropertyMarginTop *margin_top;

  g_return_if_fail (property != NULL);

  margin_top = FO_PROPERTY_MARGIN_TOP (property);

  margin_top->value = new_value;
  /*g_object_notify(G_OBJECT(margin_top), "value");*/
}

FoDatatype*
fo_property_margin_top_get_value (FoProperty *property)
{
  FoPropertyMarginTop *margin_top;

  g_return_val_if_fail (property != NULL, NULL);

  margin_top = FO_PROPERTY_MARGIN_TOP (property);

  return margin_top->value;
}

gchar*
fo_property_margin_top_sprintf (FoObject *object)
{
  FoPropertyMarginTop *margin_top;
  FoDatatype *datatype;

  g_return_val_if_fail (object != NULL, NULL);

  margin_top = FO_PROPERTY_MARGIN_TOP (object);
  datatype = fo_property_margin_top_get_value (margin_top);

  return
    g_strdup_printf ("margin-top: %s",
		     fo_object_sprintf (FO_OBJECT (datatype)));
}

FoProperty*
fo_property_margin_top_get_initial (void)
{
  FoProperty *margin_top;

  margin_top = fo_property_margin_top_new ();

  /* Set the initial value here */
  fo_property_margin_top_set_value (margin_top,
				      g_object_new (FO_TYPE_LENGTH,
						    "value",
						    0.0,
						    NULL));

  return margin_top;
}
