/* Fo
 * fo-property-page-width.c: page-width property
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-property-page-width.c,v 1.2 2003/03/21 14:41:39 tonygraham Exp $
 *
 * See Copyright for the status of this software.
 */

#include <string.h>
#include "libfo/fo-utils.h"
#include "fo-datatype.h"
#include "fo-property-private.h"
#include "fo-property-font-size.h"
#include <fo-property-page-width.h>

#define FO_PROPERTY_PAGE_WIDTH_DEFAULT (210 / 25.4) * 72

/* page-width */
/* auto | indefinite | <length> | inherit */
GType
fo_enum_page_width_get_type (void)
{
  static GType etype = 0;
  if (etype == 0)
    {
      static const GEnumValue values[] = {
        { FO_ENUM_PAGE_WIDTH_AUTO, "FO_ENUM_PAGE_WIDTH_AUTO", "auto" },
        { FO_ENUM_PAGE_WIDTH_INDEFINITE, "FO_ENUM_PAGE_WIDTH_INDEFINITE", "indefinite" },
        { FO_ENUM_PAGE_WIDTH_INHERIT, "FO_ENUM_PAGE_WIDTH_INHERIT", "inherit" },
        { 0, NULL, NULL }
      };
      etype = g_enum_register_static ("FoEnumPageWidth", values);
    }
  return etype;
}

enum {
  PROP_0,
  PROP_VALUE
};

struct _FoPropertyPageWidth
{
  FoProperty parent_instance;

  FoDatatype *value;
};

struct _FoPropertyPageWidthClass
{
  FoPropertyClass parent_class;
  
};

static void fo_property_page_width_init         (FoPropertyPageWidth      *property_page_width);
static void fo_property_page_width_base_class_init   (FoPropertyPageWidthClass *klass);
static void fo_property_page_width_base_class_finalize   (FoPropertyPageWidthClass *klass);
static void fo_property_page_width_class_init   (FoPropertyPageWidthClass *klass);
static void fo_property_page_width_set_property (GObject       *object,
				    guint          prop_id,
				    const GValue  *value,
				    GParamSpec    *pspec);
static void fo_property_page_width_get_property (GObject       *object,
				    guint          prop_id,
				    GValue        *value,
				    GParamSpec    *pspec);
static void fo_property_page_width_finalize     (GObject       *object);

FoDatatype* fo_property_page_width_copy(FoDatatype *datatype);
static FoDatatype* fo_property_page_width_resolve_enum (const gchar *token,
							 FoContext *context, GError **err);
static FoDatatype* fo_property_page_width_validate (FoDatatype *datatype,
						     FoContext *context);
static gchar*      fo_property_page_width_sprintf (FoObject *object);

static gpointer parent_class;
static FoDatatype *enum_auto;
static FoDatatype *enum_indefinite;

GType
fo_property_page_width_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoPropertyPageWidthClass),
        (GBaseInitFunc) fo_property_page_width_base_class_init,
        (GBaseFinalizeFunc) fo_property_page_width_base_class_finalize,
        (GClassInitFunc) fo_property_page_width_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoPropertyPageWidth),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_property_page_width_init,
      };
      
      object_type = g_type_register_static (FO_TYPE_PROPERTY,
                                            "FoPropertyPageWidth",
                                            &object_info, 0);
    }
  
  return object_type;
}

static void
fo_property_page_width_init (FoPropertyPageWidth *page_width)
{
  page_width->value = NULL;
}

static void
fo_property_page_width_base_class_init (FoPropertyPageWidthClass *klass)
{
  enum_auto =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_PAGE_WIDTH),
                  "value",
                  FO_ENUM_PAGE_WIDTH_AUTO,
                  NULL);
  enum_indefinite =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_PAGE_WIDTH),
                  "value",
                  FO_ENUM_PAGE_WIDTH_INDEFINITE,
                  NULL);
}

static void
fo_property_page_width_base_class_finalize (FoPropertyPageWidthClass *klass)
{  g_object_unref (enum_auto);
  g_object_unref (enum_indefinite);
}

static void
fo_property_page_width_class_init (FoPropertyPageWidthClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  
  parent_class = g_type_class_peek_parent (klass);
  
  object_class->finalize = fo_property_page_width_finalize;

  object_class->set_property = fo_property_page_width_set_property;
  object_class->get_property = fo_property_page_width_get_property;

  g_object_class_install_property (object_class,
                                   PROP_VALUE,
                                   g_param_spec_object ("value",
						     _("page-width value"),
						     _("page-width property value"),
							FO_TYPE_DATATYPE,
						     G_PARAM_READWRITE));

  FO_PROPERTY_CLASS(klass)->is_inherited = FALSE;
  FO_PROPERTY_CLASS(klass)->new_from_expression =
    fo_property_page_width_new_from_expression;
  FO_OBJECT_CLASS(klass)->sprintf =
    fo_property_page_width_sprintf;
  FO_PROPERTY_CLASS(klass)->set_value =
    fo_property_page_width_set_value;
  FO_PROPERTY_CLASS(klass)->get_value =
    fo_property_page_width_get_value;
}

static void
fo_property_page_width_finalize (GObject *object)
{
  FoPropertyPageWidth *page_width;

  page_width = FO_PROPERTY_PAGE_WIDTH (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


static void
fo_property_page_width_set_property (GObject         *object,
                         guint            prop_id,
                         const GValue    *value,
                         GParamSpec      *pspec)
{
  FoPropertyPageWidth *page_width;

  page_width = FO_PROPERTY_PAGE_WIDTH (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      fo_property_page_width_set_value (FO_PROPERTY (page_width), g_value_get_object (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
fo_property_page_width_get_property (GObject         *object,
                         guint            prop_id,
                         GValue          *value,
                         GParamSpec      *pspec)
{
  FoPropertyPageWidth *page_width;

  page_width = FO_PROPERTY_PAGE_WIDTH (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      g_value_set_instance (value,
			  fo_property_page_width_get_value (FO_PROPERTY (page_width)));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/**
 * fo_property_page_width_new:
 * 
 * Creates a new #FoPropertyPageWidth initialized to default value.
 * 
 * Return value: the new #FoPropertyPageWidth
 **/
FoProperty*
fo_property_page_width_new (void)
{
  FoProperty* page_width;

  page_width =
    FO_PROPERTY (g_object_new (fo_property_page_width_get_type (), NULL));
  
  return page_width;
}

FoDatatype*
fo_property_page_width_resolve_enum (const gchar *token, FoContext *context, GError **err)
{
  g_return_val_if_fail (token != NULL, NULL);

  if (strcmp (token, "auto") == 0)
    {
      return g_object_ref (enum_auto);
    }
  else if (strcmp (token, "indefinite") == 0)
    {
      return g_object_ref (enum_indefinite);
    }
  else if (strcmp (token, "inherit") == 0)
    {
      return g_object_ref (fo_context_get_page_width (context));
    }
  else
    {
      g_warning("Unrecognized page-width enumeration token: %s", token);
      return NULL;
    }
}

FoDatatype*
fo_property_page_width_validate (FoDatatype *datatype, FoContext *context)
{
  FoDatatype *new_datatype;
  gchar *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);

  if (FO_IS_ENUM (datatype))
    {
      if ((G_ENUM_CLASS_TYPE (fo_enum_get_enum_class (datatype))
           == FO_TYPE_ENUM_PAGE_WIDTH) &&
	  (g_enum_get_value (fo_enum_get_enum_class (datatype),
			     fo_enum_get_value (datatype)) != NULL))
	{
	  return datatype;
	}
      else
	{
	  g_object_unref (datatype);

	  return NULL;
	}
    }
  else if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype = fo_property_page_width_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype = fo_property_page_width_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else if (FO_IS_LENGTH (datatype))
    {
      return datatype;
    }
  else
    {
      g_warning("Invalid page-width datatype: %s (%s)",
	        fo_object_sprintf (FO_OBJECT (datatype)),
		g_type_name(G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      return NULL;
    }
}

FoProperty*
fo_property_page_width_new_from_expression (const gchar *expression,
					   FoContext *context,
					   FoProperty *current_font_size,
					   FoFo *fo_node,
					   GError **error)
{
  FoProperty *page_width;
  FoDatatype *datatype;

  g_return_val_if_fail (expression != NULL, NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail
    (current_font_size == NULL || FO_IS_PROPERTY_FONT_SIZE (current_font_size),
     NULL);

  page_width = fo_property_page_width_new ();

  datatype =
    fo_property_eval_expression (expression,
				 "page-width",
				 context,
				 fo_property_page_width_resolve_enum,
				 current_font_size,
				 fo_node);

  datatype = fo_property_page_width_validate (datatype, context);

  fo_property_page_width_set_value (page_width, datatype);

  return page_width;
}

/* value */
void
fo_property_page_width_set_value (FoProperty *property,
		       FoDatatype *new_value)
{
  FoPropertyPageWidth *page_width;

  g_return_if_fail (property != NULL);

  page_width = FO_PROPERTY_PAGE_WIDTH (property);

  page_width->value = new_value;
  /*g_object_notify(G_OBJECT(page_width), "value");*/
}

FoDatatype*
fo_property_page_width_get_value (FoProperty *property)
{
  FoPropertyPageWidth *page_width;

  g_return_val_if_fail (property != NULL, NULL);

  page_width = FO_PROPERTY_PAGE_WIDTH (property);

  return page_width->value;
}

gchar*
fo_property_page_width_sprintf (FoObject *object)
{
  FoPropertyPageWidth *page_width;
  FoDatatype *datatype;

  g_return_val_if_fail (object != NULL, NULL);

  page_width = FO_PROPERTY_PAGE_WIDTH (object);
  datatype = fo_property_page_width_get_value (page_width);

  return
    g_strdup_printf ("page-width: %s",
		     fo_object_sprintf (FO_OBJECT (datatype)));
}

FoProperty*
fo_property_page_width_get_initial (void)
{
  FoProperty *page_width;

  page_width = fo_property_page_width_new ();

  /* Set the initial value here */
  fo_property_page_width_set_value (page_width,
				     g_object_new (FO_TYPE_LENGTH,
						   "value",
						   FO_PROPERTY_PAGE_WIDTH_DEFAULT,
						   NULL));

  return page_width;
}
