/* Fo
 * fo-property-space-end.conditionality.c: space-end.conditionality property
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-property-space-end-conditionality.c,v 1.2 2003/03/21 14:41:43 tonygraham Exp $
 *
 * See Copyright for the status of this software.
 */

#include <string.h>
#include "libfo/fo-utils.h"
#include "fo-datatype.h"
#include "fo-length.h"
#include "fo-property-private.h"
#include "fo-property-font-size.h"
#include <fo-property-space-end-conditionality.h>

/* space-end.conditionality */
/* <space> | inherit */
GType
fo_enum_space_end_condity_get_type (void)
{
  static GType etype = 0;
  if (etype == 0)
    {
      static const GEnumValue values[] = {
        { FO_ENUM_SPACE_END_CONDITY_DISCARD, "FO_ENUM_SPACE_END_CONDITY_DISCARD", "discard" },
        { FO_ENUM_SPACE_END_CONDITY_RETAIN, "FO_ENUM_SPACE_END_CONDITY_RETAIN", "retain" },
        { 0, NULL, NULL }
      };
      etype = g_enum_register_static ("FoEnumSpaceEndCondity", values);
    }
  return etype;
}

enum {
  PROP_0,
  PROP_VALUE
};

struct _FoPropertySpaceEndCondity
{
  FoProperty parent_instance;

  FoDatatype *value;
};

struct _FoPropertySpaceEndCondityClass
{
  FoPropertyClass parent_class;
  
};

static void fo_property_space_end_condity_init         (FoPropertySpaceEndCondity      *property_space_end_condity);
static void fo_property_space_end_condity_base_class_init   (FoPropertySpaceEndCondityClass *klass);
static void fo_property_space_end_condity_base_class_finalize   (FoPropertySpaceEndCondityClass *klass);
static void fo_property_space_end_condity_class_init   (FoPropertySpaceEndCondityClass *klass);
static void fo_property_space_end_condity_set_property (GObject       *object,
				    guint          prop_id,
				    const GValue  *value,
				    GParamSpec    *pspec);
static void fo_property_space_end_condity_get_property (GObject       *object,
				    guint          prop_id,
				    GValue        *value,
				    GParamSpec    *pspec);
static void fo_property_space_end_condity_finalize     (GObject       *object);

FoDatatype* fo_property_space_end_condity_copy(FoDatatype *datatype);
static FoDatatype* fo_property_space_end_condity_resolve_enum (const gchar *token,
							 FoContext *context, GError **err);
static FoDatatype* fo_property_space_end_condity_validate (FoDatatype *datatype,
						     FoContext *context);
static gchar*      fo_property_space_end_condity_sprintf (FoObject *object);

static gpointer parent_class;
static FoDatatype *enum_discard;
static FoDatatype *enum_retain;

GType
fo_property_space_end_condity_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoPropertySpaceEndCondityClass),
        (GBaseInitFunc) fo_property_space_end_condity_base_class_init,
        (GBaseFinalizeFunc) fo_property_space_end_condity_base_class_finalize,
        (GClassInitFunc) fo_property_space_end_condity_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoPropertySpaceEndCondity),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_property_space_end_condity_init,
      };
      
      object_type = g_type_register_static (FO_TYPE_PROPERTY,
                                            "FoPropertySpaceEndCondity",
                                            &object_info, 0);
    }
  
  return object_type;
}

static void
fo_property_space_end_condity_init (FoPropertySpaceEndCondity *space_end_condity)
{
  space_end_condity->value = NULL;
}

static void
fo_property_space_end_condity_base_class_init (FoPropertySpaceEndCondityClass *klass)
{
  enum_discard =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_SPACE_END_CONDITY),
                  "value",
                  FO_ENUM_SPACE_END_CONDITY_DISCARD,
                  NULL);
  enum_retain =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_SPACE_END_CONDITY),
                  "value",
                  FO_ENUM_SPACE_END_CONDITY_RETAIN,
                  NULL);
}

static void
fo_property_space_end_condity_base_class_finalize (FoPropertySpaceEndCondityClass *klass)
{}

static void
fo_property_space_end_condity_class_init (FoPropertySpaceEndCondityClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  
  parent_class = g_type_class_peek_parent (klass);
  
  object_class->finalize = fo_property_space_end_condity_finalize;

  object_class->set_property = fo_property_space_end_condity_set_property;
  object_class->get_property = fo_property_space_end_condity_get_property;

  g_object_class_install_property (object_class,
                                   PROP_VALUE,
                                   g_param_spec_object ("value",
						     N_("space-end.conditionality value"),
						     N_("space-end.conditionality property value"),
							FO_TYPE_DATATYPE,
						     G_PARAM_READWRITE));

  FO_PROPERTY_CLASS(klass)->is_inherited = FALSE;
  FO_PROPERTY_CLASS(klass)->new_from_expression =
    fo_property_space_end_condity_new_from_expression;
  FO_OBJECT_CLASS(klass)->sprintf =
    fo_property_space_end_condity_sprintf;
  FO_PROPERTY_CLASS(klass)->set_value =
    fo_property_space_end_condity_set_value;
  FO_PROPERTY_CLASS(klass)->get_value =
    fo_property_space_end_condity_get_value;
}

static void
fo_property_space_end_condity_finalize (GObject *object)
{
  FoPropertySpaceEndCondity *space_end_condity;

  space_end_condity = FO_PROPERTY_SPACE_END_CONDITY (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


static void
fo_property_space_end_condity_set_property (GObject         *object,
                         guint            prop_id,
                         const GValue    *value,
                         GParamSpec      *pspec)
{
  FoProperty *space_end_condity;

  space_end_condity = FO_PROPERTY (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      fo_property_space_end_condity_set_value (FO_PROPERTY (space_end_condity), g_value_get_object (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
fo_property_space_end_condity_get_property (GObject         *object,
                         guint            prop_id,
                         GValue          *value,
                         GParamSpec      *pspec)
{
  FoProperty *space_end_condity;

  space_end_condity = FO_PROPERTY (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      g_value_set_instance (value,
			  fo_property_space_end_condity_get_value (FO_PROPERTY (space_end_condity)));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/**
 * fo_property_space_end_condity_new:
 * 
 * Creates a new #FoPropertySpaceEndCondity initialized to default value.
 * 
 * Return value: the new #FoPropertySpaceEndCondity
 **/
FoProperty*
fo_property_space_end_condity_new (void)
{
  FoProperty* space_end_condity;

  space_end_condity =
    FO_PROPERTY (g_object_new (fo_property_space_end_condity_get_type (), NULL));
  
  return space_end_condity;
}

FoDatatype*
fo_property_space_end_condity_resolve_enum (const gchar *token, FoContext *context, GError **err)
{
  g_return_val_if_fail (token != NULL, NULL);

  if (strcmp (token, "discard") == 0)
    {
      return g_object_ref (enum_discard);
    }
  else if (strcmp (token, "retain") == 0)
    {
      return g_object_ref (enum_retain);
    }
  else
    {
      g_warning("Unrecognized space-end.conditionality enumeration token: %s", token);
      return NULL;
    }
}

FoDatatype*
fo_property_space_end_condity_validate (FoDatatype *datatype, FoContext *context)
{
  FoDatatype *new_datatype;
  gchar *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);

  if (FO_IS_SPACE (datatype))
    {
      return datatype;
    }
  else if (FO_IS_LENGTH (datatype))
    {
      return datatype;
    }
  else if (FO_IS_ENUM (datatype))
    {
      if ((G_ENUM_CLASS_TYPE (fo_enum_get_enum_class (datatype))
           == FO_TYPE_ENUM_SPACE_END_CONDITY) &&
	  (g_enum_get_value (fo_enum_get_enum_class (datatype),
			     fo_enum_get_value (datatype)) != NULL))
	{
	  return datatype;
	}
      else
	{
	  g_object_unref (datatype);

	  return NULL;
	}
    }
  else if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype = fo_property_space_end_condity_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype = fo_property_space_end_condity_resolve_enum (token, context, NULL);

      g_object_unref (datatype);

      return new_datatype;
    }
  else
    {
      g_warning("Invalid space-end.conditionality datatype: %s (%s)",
	        fo_object_sprintf (FO_OBJECT (datatype)),
		g_type_name(G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      return NULL;
    }
}

FoProperty*
fo_property_space_end_condity_new_from_expression (const gchar *expression,
					   FoContext *context,
					   FoProperty *current_font_size,
					   FoFo *fo_node,
					   GError **error)
{
  FoProperty *space_end_condity;
  FoDatatype *datatype;

  g_return_val_if_fail (expression != NULL, NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail
    (current_font_size == NULL || FO_IS_PROPERTY_FONT_SIZE (current_font_size),
     NULL);

  space_end_condity = fo_property_space_end_condity_new ();

  datatype =
    fo_property_eval_expression (expression,
				 "space-end.conditionality",
				 context,
				 fo_property_space_end_condity_resolve_enum,
				 current_font_size,
				 fo_node);

  datatype = fo_property_space_end_condity_validate (datatype, context);

  fo_property_space_end_condity_set_value (space_end_condity, datatype);

  return space_end_condity;
}

/* value */
void
fo_property_space_end_condity_set_value (FoProperty *property,
		       FoDatatype *new_value)
{
  FoPropertySpaceEndCondity *space_end_condity;

  g_return_if_fail (property != NULL);

  space_end_condity = FO_PROPERTY_SPACE_END_CONDITY (property);

  space_end_condity->value = new_value;
  /*g_object_notify(G_OBJECT(space_end_condity), "value");*/
}

FoDatatype*
fo_property_space_end_condity_get_value (FoProperty *property)
{
  FoPropertySpaceEndCondity *space_end_condity;

  g_return_val_if_fail (property != NULL, NULL);

  space_end_condity = FO_PROPERTY_SPACE_END_CONDITY (property);

  return space_end_condity->value;
}

gchar*
fo_property_space_end_condity_sprintf (FoObject *object)
{
  FoProperty *space_end_condity;
  FoDatatype *datatype;

  g_return_val_if_fail (object != NULL, NULL);

  space_end_condity = FO_PROPERTY (object);
  datatype = fo_property_space_end_condity_get_value (space_end_condity);

  return
    g_strdup_printf ("space-end.conditionality: %s",
		     fo_object_sprintf (FO_OBJECT (datatype)));
}

FoProperty*
fo_property_space_end_condity_get_initial (void)
{
  static FoProperty *space_end_condity = NULL;

  if (!space_end_condity)
    {
      space_end_condity = fo_property_space_end_condity_new ();

      fo_property_space_end_condity_set_value (space_end_condity,
							 enum_discard);
    }

  return space_end_condity;
}
